import 'package:badges/badges.dart';
import 'package:flutter/material.dart';
// ignore: import_of_legacy_library_into_null_safe
import 'package:flutter_icons/flutter_icons.dart';
import 'package:modular_system_core/mscore.dart';

class EntityTableAppBar<T> extends StatefulWidget {
  // ignore: use_key_in_widget_constructors
  const EntityTableAppBar(this.viewModel);

  final EntityTableViewModel<T> viewModel;
  TableState get state {
    return viewModel.state;
  }

  @override
  State<EntityTableAppBar> createState() {
    return _EntityTableAppBarState<T>();
  }
}

class _EntityTableAppBarState<T> extends State<EntityTableAppBar<T>> {
  late bool isLoading = widget.state.isLoading;
  late bool isSelectorOn = widget.state.isSelectorOn;
  late int filtersCount = widget.viewModel.state.activeFilters();
  late TextEditingController controller = TextEditingController();
  late PaginationIn paginationIn = widget.state.paginationIn;
  late PaginationOut paginationOut = widget.state.paginationOut;

  @override
  void initState() {
    subscribeToEvents();
    super.initState();
  }

  @override
  void dispose() {
    unsubscribeToEvents();
    super.dispose();
  }

  void subscribeToEvents() {
    widget.state.subscribe(onStateChanged);
  }

  void unsubscribeToEvents() {
    widget.state.unsubscribe(onStateChanged);
  }

  void onStateChanged(TableState state) {
    setState(() {
      isLoading = state.isLoading;
      isSelectorOn = state.isSelectorOn;
      filtersCount = state.activeFilters();
      controller.text = state.searchText;
      controller.selection = TextSelection.fromPosition(TextPosition(offset: controller.text.length));
      paginationIn = state.paginationIn;
      paginationOut = state.paginationOut;
    });
  }

  String getPagination() {
    var currentPage = paginationOut.offset / paginationOut.limit + 1;
    var totalPages = paginationOut.total / paginationOut.limit + 1;
    var total = "$currentPage de ${totalPages.floor()}";
    return total;
  }

  bool nextPageExists() {
    return paginationOut.total > (paginationOut.offset + paginationOut.limit);
  }

  bool previousPageExists() {
    return paginationOut.offset > 0;
  }

  // !try adding an app bar inside the bottom section of the sliver app bar!
  @override
  Widget build(BuildContext context) {
    return SliverAppBar(
      pinned: true,
      automaticallyImplyLeading: false,
      elevation: 10,
      backgroundColor: widget.viewModel.appViewModel.colors.background,
      title: TextField(
        controller: controller,
        style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
        decoration: InputDecoration(
          hintText: "Pesquisar ...",
          hintStyle: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
          focusedBorder: UnderlineInputBorder(
            borderSide: BorderSide(
              color: widget.viewModel.appViewModel.colors.onPrimary,
            ),
          ),
          border: InputBorder.none,
        ),
        onChanged: widget.viewModel.setSearchText,
      ),
      leading: IconButton(
        icon: Icon(
          Icons.search,
          color: widget.viewModel.appViewModel.colors.onPrimary,
          size: widget.viewModel.appViewModel.fontSize.subtitle,
        ),
        onPressed: () {
          widget.viewModel.update();
        },
      ),
      actions: [
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: widget.viewModel.appViewModel.fontSize.subtitle * 0.5,
          ),
          icon: Icon(
            Icons.clear,
            color: widget.viewModel.appViewModel.colors.onPrimary,
            size: widget.viewModel.appViewModel.fontSize.subtitle,
          ),
          onPressed: () {
            widget.viewModel.clearAllSearchFilters();
            widget.viewModel.update();
          },
        ),
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: widget.viewModel.appViewModel.fontSize.subtitle * 0.5,
          ),
          icon: Icon(
            Icons.post_add,
            color: widget.viewModel.appViewModel.colors.onPrimary,
            size: widget.viewModel.appViewModel.fontSize.subtitle,
          ),
          onPressed: widget.viewModel.openForm,
        ),
        IconButton(
          padding: EdgeInsets.symmetric(
            horizontal: widget.viewModel.appViewModel.config.fontSize.subtitle * 0.5,
          ),
          icon: Icon(
            MaterialCommunityIcons.restore,
            color: widget.viewModel.appViewModel.colors.onPrimary,
            size: widget.viewModel.appViewModel.fontSize.subtitle,
          ),
          onPressed: widget.viewModel.update,
        ),
      ],
      bottom: AppBar(
        automaticallyImplyLeading: false,
        elevation: 0,
        backgroundColor: widget.viewModel.appViewModel.colors.background,
        title: Container(
          alignment: Alignment.center,
          //color: Colors.green,
          child: Row(
            mainAxisAlignment: MainAxisAlignment.spaceBetween,
            crossAxisAlignment: CrossAxisAlignment.center,
            children: [
              Row(
                children: [
                  Badge(
                    position: BadgePosition.topEnd(),
                    badgeColor: widget.viewModel.appViewModel.colors.tertiary,
                    badgeContent: Text(
                      filtersCount.toString(),
                      style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onTertiary, FontSize.label),
                    ),
                    child: IconButton(
                      icon: Icon(
                        Icons.filter_list,
                        color: widget.viewModel.appViewModel.colors.onBackground,
                        size: widget.viewModel.appViewModel.fontSize.subtitle,
                      ),
                      onPressed: widget.viewModel.openFilterSheet,
                    ),
                  ),
                  VerticalDivider(color: widget.viewModel.appViewModel.colors.onBackground),
                  IconButton(
                    icon: Icon(
                      Icons.check_box,
                      color: widget.viewModel.state.isSelectorOn
                          ? widget.viewModel.appViewModel.colors.tertiary
                          : widget.viewModel.appViewModel.colors.onBackground,
                      size: widget.viewModel.appViewModel.fontSize.subtitle,
                    ),
                    onPressed: widget.viewModel.flipSelector,
                  ),
                  VerticalDivider(color: widget.viewModel.appViewModel.colors.onBackground),
                  actionsDropdown(),
                  VerticalDivider(color: widget.viewModel.appViewModel.colors.onBackground),
                  actions(),
                  paginationInfo(),
                ],
              ),
              Row(
                mainAxisAlignment: MainAxisAlignment.end,
                children: [
                  paginationButtons(),
                  searchLimitSelector(),
                ],
              ),
            ],
          ),
        ),
      ),
    );
  }

  Widget actionsDropdown() {
    return PopupMenuButton(
      icon: Icon(
        Icons.more_vert,
        color: widget.viewModel.appViewModel.colors.onBackground,
        size: widget.viewModel.appViewModel.fontSize.subtitle,
      ),
      itemBuilder: (context) {
        var items = <PopupMenuItem<int>>[];
        var actions = widget.viewModel.actions;
        int index = 0;

        if (actions.isNotEmpty) {
          for (index = 0; index < actions.length; index++) {
            var action = actions.elementAt(index);
            var item = PopupMenuItem(
              value: index,
              child: AppbarAction(viewModel: widget.viewModel, action: action),
              onTap: () => action.action(widget.viewModel),
            );
            items.add(item);
          }
        }

        items.add(PopupMenuItem(
          value: index,
          child: AppbarAction(
            viewModel: widget.viewModel,
            action: DataTableAction(
              label: "Cancelar",
              icon: Icons.cancel,
              action: (viewModel) {},
            ),
          ),
          onTap: () {},
        ));
        return items;
      },
    );
  }

  Widget actions() {
    var children = <Widget>[];
    for (var action in widget.viewModel.actions) {
      children.add(
        IconButton(
          icon: Icon(
            action.icon,
            color: widget.viewModel.appViewModel.colors.onBackground,
            size: widget.viewModel.appViewModel.fontSize.subtitle,
          ),
          onPressed: () {
            action.action(widget.viewModel);
          },
        ),
      );
    }
    children.add(const VerticalDivider());
    return Row(
      children: children,
    );
  }

  Widget paginationInfo() {
    getPagination();
    return Row(
      children: [
        Padding(
          padding: const EdgeInsets.symmetric(horizontal: 5),
          child: Icon(
            MaterialCommunityIcons.table_settings,
            color: widget.viewModel.appViewModel.colors.onBackground,
            size: widget.viewModel.appViewModel.fontSize.subtitle,
          ),
        ),
        Text(
          "${getPagination()} Total: ${paginationOut.total}",
          style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
        )
      ],
    );
  }

  Widget paginationButtons() {
    if (isLoading) {}
    return Container(
      margin: const EdgeInsets.symmetric(horizontal: 25),
      child: Row(
        children: [
          IconButton(
            iconSize: widget.viewModel.appViewModel.fontSize.subtitle,
            mouseCursor: previousPageExists() ? null : SystemMouseCursors.basic,
            splashRadius: previousPageExists() ? null : 1,
            color: previousPageExists() ? widget.viewModel.appViewModel.colors.tertiary : widget.viewModel.appViewModel.colors.onBackground,
            padding: const EdgeInsets.symmetric(horizontal: 15),
            icon: const Icon(Icons.navigate_before),
            onPressed: widget.viewModel.navigateBackward,
          ),
          IconButton(
            iconSize: widget.viewModel.appViewModel.fontSize.subtitle,
            mouseCursor: nextPageExists() ? null : SystemMouseCursors.basic,
            splashRadius: nextPageExists() ? null : 1,
            color: nextPageExists() ? widget.viewModel.appViewModel.colors.tertiary : widget.viewModel.appViewModel.colors.onBackground,
            padding: const EdgeInsets.symmetric(
              horizontal: 15,
            ),
            icon: const Icon(
              Icons.navigate_next,
            ),
            onPressed: widget.viewModel.navigateForward,
          ),
        ],
      ),
    );
  }

  Widget searchLimitSelector() {
    return DropdownButton<int>(
      value: paginationIn.limit,
      focusColor: Colors.transparent,
      dropdownColor: widget.viewModel.appViewModel.colors.primary,
      iconEnabledColor: widget.viewModel.appViewModel.colors.onBackground,
      underline: SizedBox(
        height: 1.5,
        child: Container(
          color: widget.viewModel.appViewModel.colors.onBackground,
        ),
      ),
      items: <DropdownMenuItem<int>>[
        DropdownMenuItem(
          value: 10,
          child: Text(
            "10",
            style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
          ),
        ),
        DropdownMenuItem(
          value: 30,
          child: Text(
            "30",
            style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
          ),
        ),
        DropdownMenuItem(
          value: 50,
          child: Text(
            "50",
            style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
          ),
        ),
        DropdownMenuItem(
          value: 100,
          child: Text(
            "100",
            style: widget.viewModel.appViewModel.styleProvider.auto(FontType.onBackground, FontSize.subtitle),
          ),
        ),
      ],
      onChanged: (limit) {
        widget.viewModel.setPaginationInLimit(limit!);
        widget.viewModel.update();
      },
    );
  }
}

class AppbarAction extends StatelessWidget {
  const AppbarAction({
    Key? key,
    required this.viewModel,
    required this.action,
  }) : super(key: key);

  final EntityTableViewModel viewModel;
  final DataTableAction action;

  @override
  Widget build(BuildContext context) {
    return Row(
      children: [
        Icon(
          action.icon,
          color: viewModel.appViewModel.colors.onBackground,
          size: viewModel.appViewModel.fontSize.data,
        ),
        const VerticalDivider(),
        Text(
          action.label,
          style: viewModel.appViewModel.styleProvider.auto(FontType.onBackground),
        ),
      ],
    );
  }
}
