import 'package:flutter/material.dart';
import 'package:modular_system_core/mscore.dart';

class TableState<T> extends ReactiveObject<TableState> {
  TableState({
    required this.body,
  });

  bool isLoading = false;
  bool isSelectorOn = false;
  String searchText = "";
  Widget body;
  PaginationIn paginationIn = PaginationIn.fromLimit(30);
  PaginationOut paginationOut = PaginationOut.fromLimit(30);
  List<int> selectedTiles = <int>[];
  List<WebSearchFilter> filters = <WebSearchFilter>[];
  List<T> data = <T>[];

  int activeFilters() {
    return filters.where((element) => element.isActive).toList().length;
  }
}

class EntityTableViewModel<T> {
  EntityTableViewModel({
    required this.context,
    required this.appViewModel,
    required this.actions,
    required this.onUpdate,
    required this.itemBuilder,
    this.itemViewerBuilder,
    this.editorBuilder,
    required this.formBuilder,
    required this.filtersSheetBuilder,
  }) {
    state.subscribe(onStateChanged);
  }

  static const String textFilterName = "";

  // injected
  final BuildContext context;
  final AppViewModel appViewModel;
  final List<DataTableAction> actions;
  // callbacks
  final Future<KeyValuePair<PaginationOut, List<T>>> Function(WebSearchRequest search) onUpdate;
  // builders
  final IteratorBuilder<T> itemBuilder;
  final IteratorBuilder<T>? itemViewerBuilder;
  final IteratorBuilder<T>? editorBuilder;
  final ScreenBuilder<T> formBuilder;
  final ScreenBuilder<T> filtersSheetBuilder;
  // inner state
  late TableState<T> state = TableState(body: defaultWidget());
  // shortcuts
  ColorScheme get colors {
    return appViewModel.colors;
  }

  FontSizeScheme get fontSize {
    return appViewModel.fontSize;
  }

  IStyleProvider get styleProvider {
    return appViewModel.styleProvider;
  }

  IErrorHandler get errorHandler {
    return appViewModel.errorHandler;
  }

  IInformer get informer {
    return appViewModel.informer;
  }

  ILoader get loader {
    return appViewModel.loader;
  }

  Widget defaultWidget() {
    return EntityTableBody<T>(this);
  }

  // state change related
  void onStateChanged(TableState state) {
    _handleSearchFilterStateChange(state);
  }

  void _handleSearchFilterStateChange(TableState state) {
    state.filters.removeWhere((element) => element.name == textFilterName);
    if (state.searchText.isNotEmpty) {
      state.filters.add(getSearchTextFilter(state.searchText.trim()));
    }
  }

  //
  WebSearchFilter getSearchTextFilter(String value) {
    return WebSearchFilter(name: textFilterName, value: value, isActive: true);
  }

  // search related
  void update() async {
    try {
      setLoading(true);
      var data = await onUpdate.call(WebSearchRequest.build(state.paginationIn, state.filters));
      setData(data);
      setLoading(false);
    } catch (e) {
      appViewModel.errorHandler.handle(e);
    }
  }

  void setData(KeyValuePair<PaginationOut, List<T>> data) {
    state.change((state) {
      state.paginationOut = data.key;
      state.data = data.value;
    });
  }

  void clearAllSearchFilters() {
    clearSearchText();
  }

  void clearSearchFilters() {
    state.change((state) {
      state.filters.clear();
    });
  }

  void clearSearchText() {
    state.change((state) {
      state.searchText = "";
    });
  }

  bool nextPageExists() {
    return state.paginationOut.total > (state.paginationOut.offset + state.paginationOut.limit);
  }

  bool previousPageExists() {
    return state.paginationOut.offset > 0;
  }

  void navigateForward() {
    if (nextPageExists()) {
      state.paginationIn.offset = state.paginationIn.offset + state.paginationIn.limit;
      update();
    }
  }

  void navigateBackward() {
    if (previousPageExists()) {
      state.paginationIn.offset = state.paginationIn.offset - state.paginationIn.limit;
      update();
    }
  }

  void setPaginationInLimit(int limit) {
    state.change((state) {
      state.paginationIn.limit = limit;
    });
  }

  void setSearchText(String text) {
    state.change((state) {
      state.searchText = text;
    });
  }

  // widget navigation
  void openTable() {
    setBody(EntityTableBody(this));
  }

  void openForm() {
    setBody(formBuilder(this));
  }

  void openFilterSheet() {
    showBottomSheet(context: context, builder: (context) => filtersSheetBuilder(this));
  }

  void openItemViewer(T data) {
    if (itemViewerBuilder != null) setBody(itemViewerBuilder!.call(this, data));
  }

  void openEditor(T data) {
    if (editorBuilder != null) setBody(editorBuilder!.call(this, data));
  }

  // state related
  void setBody(Widget body) {
    state.change((state) {
      state.body = body;
    });
  }

  void setLoading(bool toggle) {
    state.change((state) {
      state.isLoading = toggle;
    });
  }

  void toggleSelector(bool toggle) {
    state.change((state) {
      state.isSelectorOn = toggle;
    });
  }

  void flipSelector() {
    state.change((state) {
      state.isSelectorOn = !state.isSelectorOn;
    });
  }

  void selectTile(int index) {
    state.change((state) {
      state.selectedTiles.add(index);
    });
  }

  void unselectTile(int index) {
    state.change((state) {
      state.selectedTiles.remove(index);
    });
  }
}

class DataTableAction {
  DataTableAction({required this.label, required this.icon, required this.action});

  String label;
  IconData icon;
  void Function(EntityTableViewModel viewModel) action;
}
